/**
 * storage.provider.ts
 * Capa provider del almacenamiento de datos persistentes.
 * Android: se almacena en el SharedPreferences.
 * iOS: se almacena en el NSUserDefaults.
 * NO SE BORRA: 
 * - al contrario que el localStorage, al borrar la caché del navegador, estos datos no se borran.
 * - tampoco al reinstalar la aplicación o sobrescribirla.
 * 
 * SÍ SE BORRA:
 * - al desinstalar la aplicación.
 * - al borrar manualmente los datos de la aplicación.
 * 
 * ACTUALIZACION IMPORTANTE v2.0:
 * Ver flujo de Trello. Consiste en que al hacer el get, vemos si hay datos en el local
 * storage (las versiones anteriores a esta que puedan tener datos o navegador).
 * Si tiene datos, los vuelco al storage nativo y borro los del navegador.
 * 
 * v3.0: se crean los métodos de almacenado y carga rápida de memoria del navegador setLocalData y getLocalData
 */
angular.module('selae.core.services')
.factory('storageProvider', ['$q', function($q){

    //Realmente esto está acoplado a la lógica de Negocio y no a un proveedor. De momento aquí.
    this.KEY_STORAGE_VERSION = 'version';
    this.KEY_STORAGE_PROPERIESVERSION = 'propertiesVersion';
    this.KEY_STORAGE_PROPERTIES = 'properties';
    this.KEY_STORAGE_LEGALAD = 'LegalAdviceAccepted';
    this.KEY_STORAGE_PUSHENABLED = 'receiveNotifications';
    this.KEY_STORAGE_FIRSTLOGIN = 'isFirstLogin';
    this.KEY_STORAGE_GAMES = 'games';
    this.KEY_STORAGE_USER = 'SelaeUsername';
    this.KEY_STORAGE_CLAVE = 'clave';
    this.KEY_LOCALSTORAGE_STATICCARROUSEL = 'keysStaticcarrousel';
    this.KEY_UUID = 'UUID';
    

    /**
     * Guarda los datos de value con la clave de 'key'.
     * Devuelve null si error y el valor almacenado si éxito.
     * @param in {string} key: clave bajo la cual se guarda la entrada.
     * @param in {*} value: valor a guardar.
     * @param out {*}: promesa con el dato o ko.
     */
    this.setData = function(key, value){
        var deferred = $q.defer();

        //Para poder seguir trabajando en Chrome, se realiza la siguiente confirmación
        if(window.cordova){
            if(ionic.Platform.isAndroid()){
                //Android Shared Preferences tiene un tipado fuerte que hace que falle con JS
                try{
                    value = '' + value;
                }catch(err){
                    console.log('Fracaso al convertir ' + value);
                    console.log(err);
                }
            }
            NativeStorage.set(key, value,
                function (result) {
                    console.log('EXITO EN SET ('+ key +') -> ' + result);
                    deferred.resolve(result);
                },
                function (e) {
                    console.error('storage.provider error in set: ' + e);
                    deferred.reject(e);
                });
        }else{
            try{
                window.localStorage.setItem(key, value);
                deferred.resolve(value);
            }catch(e){
                console.error('storage.provider error in set: ' + e);
                deferred.reject(e);
            }
        }
                           
        return deferred.promise;
    };

    /**
     * Retorna los datos guardados bajo la clave 'key'.
     * @param in {*} key: clave bajo la cual se guarda la entrada.
     * @param out {*}: promesa con el dato o ko.
     */
    this.getData = function(key){
        var deferred = $q.defer();

        //Para poder seguir trabajando en Chrome, se realiza la siguiente confirmación
        if(window.cordova){
            //MIGRACION DE DATOS:
            //El terminal es real, ahora bien, tiene datos del navegador?
            var tempResult = window.localStorage.getItem(key);
            if(tempResult !== undefined && tempResult != undefined && tempResult !== 'undefined' && tempResult != null){
                console.log('Migración de local a native storage: ' + key);
                //Pues se tienen datos, probablemente de versiones anteriores:
                //1) lo guardo en el storage nativo.
                NativeStorage.set(key, tempResult,
                    function (result) {
                        console.log('EXITO EN SET ('+ key +') -> ' + result);
                        //1.bis) ahora que los tenemos en el nativo, los borro del local.
                        window.localStorage.setItem(key, undefined);
                    },
                    function (e) {
                        console.error('storage.provider error in set: ' + e);
                        //aquí no se borra nada del nativo. 
                        //Quedaría en local si no se puede guardar.
                    });
                
                //2) devuelvo el dato a la capa superior.
                deferred.resolve(tempResult);
            }else{
                //No hay datos: probablemente nueva descarga o borrado de datos.
                NativeStorage.getString(key,
                    function (result) {
                        console.log('EXITO EN GET ('+ key +') -> ' + result);
                        if (result === 'undefined' || result === 'null') {
                            result = undefined;
                        }
                        if ( result === 'false' ) { result = false; }
                        deferred.resolve(result);
                    },
                    function (e) {
                        console.error('storage.provider error in get: ' + e);
                        deferred.reject(e);
                    });
            }
        }else{
            try{
                var result = window.localStorage.getItem(key);
                deferred.resolve(result);
            }catch(e){
                console.error('storage.provider error in get: ' + e);
                deferred.reject(e);
            }
        }
        
        return deferred.promise;
    };

    /**
     * Guarda los datos de value con la clave de 'key'.
     * Devuelve null si error y el valor almacenado si éxito.
     * @param in {string} key: clave bajo la cual se guarda la entrada.
     * @param in {*} value: valor a guardar.
     * @param out {*}: promesa con el dato o ko.
     */
    this.setLocalData = function(key, value){
        var result = null;

        try{
            window.localStorage.setItem(key, value);
            result = value;
        } catch (err) {
            console.log('Fracaso en guardar ('+ key +') -> ' + err);
        }

        return result;
    };

    /**
     * Retorna los datos guardados bajo la clave 'key'.
     * @param in {*} key: clave bajo la cual se guarda la entrada.
     * @param out {*}: promesa con el dato o ko.
     */
    this.getLocalData = function(key){
        var result = null;

        try{
            result = window.localStorage.getItem(key);
        } catch (err) {
            console.log('Fracaso en extraer ('+ key +') -> ' + err);
        }

        return result;
    };

    return{
        KEY_STORAGE_VERSION: this.KEY_STORAGE_VERSION,
        KEY_STORAGE_PROPERIESVERSION: this.KEY_STORAGE_PROPERIESVERSION,
        KEY_STORAGE_PROPERTIES: this.KEY_STORAGE_PROPERTIES,
        KEY_STORAGE_CLAVE: this.KEY_STORAGE_CLAVE,
        KEY_STORAGE_LEGALAD: this.KEY_STORAGE_LEGALAD,
        KEY_STORAGE_PUSHENABLED: this.KEY_STORAGE_PUSHENABLED,
        KEY_STORAGE_FIRSTLOGIN: this.KEY_STORAGE_FIRSTLOGIN,
        KEY_STORAGE_GAMES: this.KEY_STORAGE_GAMES,
        KEY_STORAGE_USER: this.KEY_STORAGE_USER,
        KEY_LOCALSTORAGE_STATICCARROUSEL: this.KEY_LOCALSTORAGE_STATICCARROUSEL,
        KEY_UUID: this.KEY_UUID,
        setData: this.setData,
        getData: this.getData,
        setLocalData: this.setLocalData,
        getLocalData: this.getLocalData
    };
}]);
